Emulation of LocalFileSystem with virtual COM.

Dependencies:   USBDevice

Dependents:   KL46Z-lpc81isp lpcterm2

#include "USBLocalFileSystem.h"

int main() {
    USBLocalFileSystem* usb_local = new USBLocalFileSystem(); // RamDisk(64KB)

    while(1) {
        usb_local->lock(true);
        usb_local->remount();
        char filename[32];
        if (usb_local->find(filename, sizeof(filename), "*.TXT")) {
            FILE* fp = fopen(filename, "r");
            if (fp) {
                int c;
                while((c = fgetc(fp)) != EOF) {
                    usb_local->putc(c);
                }
                fclose(fp);
            }
        }    
        usb_local->lock(false);

        wait_ms(1000*5);
    }
}



Sample application:

Import programKL46Z-lpc81isp

ISP example program.

Import programlpcterm2

semihost server example program

Revision:
4:8f6857784854
Parent:
0:39eb4d5b97df
--- a/src/Storage.cpp	Tue May 06 16:05:57 2014 +0900
+++ b/src/Storage.cpp	Thu May 08 23:43:46 2014 +0900
@@ -1,7 +1,5 @@
 #include "Storage.h"
 #include "FATFileSystem.h"
-#include "mbed_debug.h"
-#include "mystring.h"
 #include <ctype.h>
 
 #if (DEBUG2 > 3)
@@ -10,18 +8,52 @@
 #define STORAGE_DBG(...)
 #endif
 
-#define LOCAL_DIR "local"
-
 LocalStorage::LocalStorage(StorageInterface* storage, const char* name)
         : FATFileSystem(name),_storage(storage)
 {
     _name = name;
 }
 
-extern FILINFO FATDirHandle_finfo; // fst/FATDirHandle.cpp
-/* static */ bool LocalStorage::find_bin(mystring& filename)
+static bool match1(const char* name, const char* pat) {
+	while(1) {
+    	char c = *pat++;
+    	char d = *name++;
+        if (c == '\0' && d == '\0') {
+        	return true;
+        } else if (c == '\0' || d == '\0') {
+        	return false;
+        }
+    	switch(c) {
+    	    case '?':
+    	    	break;
+    	    case '*':
+    	    	name--;
+    	    	while(*name) {
+    	    		if (*name == '.') {
+    	    			break;
+    	    		}
+    	    		name++;
+    	    	}
+    	    	break;
+    	    default:
+    	    	if (toupper(d) != toupper(c)) {
+    	    		return false;
+    	    	}
+    	    	break;
+    	}
+    }
+}
+
+extern FILINFO FATDirHandle_finfo; // fat/FATDirHandle.cpp
+/* static */ bool LocalStorage::find(char* name, size_t size, const char* dirname, const char* pat)
 {
-    DIR *dir = ::opendir("/"LOCAL_DIR);
+    char dirpath[32];
+    strcpy(dirpath, "/");
+    if (strlen(dirname) >= sizeof(dirpath)-2) {
+    	return false;
+    }
+    strcat(dirpath, dirname);
+    DIR *dir = ::opendir(dirpath);
     if (dir == NULL) {
         return false;
     }
@@ -29,23 +61,22 @@
     bool found = false;
     struct dirent *entry;
     while ((entry = readdir(dir)) != NULL) {
-        mystring name(entry->d_name);
-        int len = name.size();
-        if (name[len-4] == '.' && 
-            toupper(name[len-3]) == 'B' &&
-            toupper(name[len-2]) == 'I' &&
-            toupper(name[len-1]) == 'N') {
+        if (match1(entry->d_name, pat)) {
             FILINFO* fi = &FATDirHandle_finfo;
             uint32_t datetime =  fi->ftime | (fi->fdate<<16);
             STORAGE_DBG("datetime=%08x [%s]", datetime, entry->d_name);
             if (datetime > fdatetime) {
                 fdatetime = datetime;
-                filename = "/"LOCAL_DIR"/";
-                filename += entry->d_name;
-                found = true;
+                if (strlen(dirpath) + 1 + strlen(entry->d_name) < size) {
+                    strcpy(name, dirpath);
+                    strcat(name, "/");
+                    strcat(name, entry->d_name);
+                    found = true;
+                }
             }
         }
     }
     closedir(dir);
     return found;
 }
+